from enum import Enum


def map_entity_tags(root):
    return {e.tag: e.text for e in root.findall('./entities/entity/*')}


class Object(object):
    def __init__(self, client, oid, root=None):
        if root is None:
            root = client.get(client.url('basemetadata/digitalObjects', oid))

        entities = map_entity_tags(root)

        self.client = client
        self.id = int(entities['baseId'])
        self.uuid = entities['digitalObjectIdentifier']
        self.investigation_id = root.find('entities/entity/investigation/investigationId').text
        self.label = entities.get('label')
        self.note = entities.get('label')

        node = root.find('entities/entity/uploader/userId')
        self.uploader = node.text if node is not None else None

    def __repr__(self):
        return '<Object:id={} uuid={} label={} note={}>'.format(self.id, self.uuid, self.label, self.note)

    def create_ingest(self, accesspoint):
        url = self.client.url('staging/ingests')
        return Ingest(self.client, -1, root=self.client.post(url, objectId=self.uuid, accessPoint=accesspoint.uuid))

    def create_download(self, accesspoint):
        url = self.client.url('staging/downloads')
        return Download(self.client.post(url, objectId=self.id, accessPoint=accesspoint.uuid))


class Credentials(object):
    def __init__(self, root):
        entities = {e.find('key').text: e.find('value').text for e in root.find("./entities")}
        self.user = entities['user.id']
        self.key = entities['rest.key']
        self.secret = entities['rest.secret']


class IngestStatus(Enum):
    UNKNOWN = 0
    PREPARING = 1
    PREPARATION_FAILED = 2
    PRE_INGEST_SCHEDULED = 4
    PRE_INGEST_RUNNING = 8
    PRE_INGEST_FINISHED = 16
    PRE_INGEST_FAILED = 32
    INGEST_RUNNING = 64
    INGEST_FINISHED = 128
    INGEST_FAILED = 256
    INGEST_REMOVED = 512


class Ingest(object):
    def __init__(self, client, oid, root=None):
        if root is None:
            root = client.get(client.url('staging/ingests', oid))

        entities = map_entity_tags(root)
        self.client = client
        self.id = int(entities.get('id'))
        self.staging_url = entities.get('stagingUrl')
        self.object_uuid = entities.get('digitalObjectUuid')
        self.owner_uuid = entities.get('owernUuid')

    def __repr__(self):
        return '<Ingest:id={} object_uuid={} url={} status={}>'.format(self.id, self.object_uuid, self.staging_url, self.status)

    @property
    def status(self):
        entities = map_entity_tags(self.client.get(self.client.url('staging/ingests', self.id)))
        return IngestStatus(int(entities.get('status', 0)))

    @status.setter
    def status(self, value):
        if not isinstance(value, IngestStatus):
            raise TypeError

        params = dict(status=value.value, errorMessage='')
        url = self.client.url('staging/ingests', self.id)

        self.client.put(url, data=params)


class Investigation(object):
    def __init__(self, client, oid):
        root = client.get(client.url('basemetadata/investigations', oid))
        entities = map_entity_tags(root)
        self.id = oid
        self.object_ids = [e.text for e in root.findall('./entities/entity/digitalObjects/digitalObject/baseId')]
        self.uuid = entities.get('uniqueIdentifier')
        self.client = client

    def create_object(self, uploader, label=None, note=None):
        return self.client.create_object(self.id, uploader, label=label, note=note)


class AccessPoint(object):
    def __init__(self, client, oid):
        entities = map_entity_tags(client.get(client.url('staging/accesspoints', oid)))
        self.id = oid
        self.url = entities['remoteBaseUrl']
        self.uuid = entities['uniqueIdentifier']

    def __repr__(self):
        return '<AccessPoint:id={} uuid={} url={}>'.format(self.id, self.uuid, self.url)


class User(object):
    def __init__(self, client, oid):
        root = client.get(client.url('usergroup/users', oid))
        entities = map_entity_tags(root)
        self.client = client
        self.id = int(entities['userId'])
        self.ldap_id = int(entities['distinguishedName'])

        url = self.client.url('information/properties')
        response = client.get(url, userId=self.ldap_id)
        self.credentials = Credentials(response)

    def __repr__(self):
        return '<User:id={} ldap_id={}>'.format(self.id, self.ldap_id)


class Download(object):
    def __init__(self, client, oid):
        root = client.get(client.url('staging/downloads', oid))
        entities = map_entity_tags(root)
        self.id = int(entities['id'])
        self.object_uuid = entities['digitalObjectUuid']
        self.owner_uuid = entities['ownerUuid']
        self.status = entities['status']
        self.url = entities['stagingUrl']

    def __repr__(self):
        return '<Download:id={} object_uuid={} url={}>'.format(self.id, self.object_uuid, self.url)
